#! /usr/bin/env python

import os, sys, subprocess, argparse, platform

parser = argparse.ArgumentParser()
parser.add_argument('-c', '--compiler', choices=['gnu', 'intel'], default='gnu')
args = parser.parse_args()

e = os.environ.copy()

if args.compiler == 'intel':
  e['CC'] = 'icx'
  e['CXX'] = 'icpx'
else:
  e['CC'] = 'gcc'
  e['CXX'] = 'g++'

base = os.getcwd()
srcDir = os.path.join(base, 'src')
buildDir = os.path.join(base, 'build')
installDir = os.path.join(base, 'install')

if not os.path.exists(buildDir):
  os.makedirs(buildDir)

configureCmd = ['cmake', '-DCMAKE_INSTALL_PREFIX=' + installDir, srcDir]
print(' '.join(configureCmd))
err = subprocess.call(configureCmd, cwd=buildDir, env=e)

if err == 0:
  compileCmd = ['make', '--no-print-directory', 'install']
  print(' '.join(configureCmd))
  err = subprocess.call(compileCmd, cwd=buildDir, env=e)

